<?php

namespace Drupal\ckeditor_translator\Plugin\CKEditorPlugin;

use Drupal\ckeditor\CKEditorPluginBase;
use Drupal\ckeditor\CKEditorPluginConfigurableInterface;
use Drupal\ckeditor\CKEditorPluginContextualInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\editor\Entity\Editor;

/**
 * Defines the "CKEditor Translator" plugin.
 *
 * @CKEditorPlugin(
 *   id = "jsplusTranslator",
 *   label = @Translation("CKEditor Translator"),
 *   module = "ckeditor_translator"
 * )
 */
class ckeditor_translator extends CKEditorPluginBase implements CKEditorPluginConfigurableInterface, CKEditorPluginContextualInterface {

  public $plugins = array(
     'jsplusTranslator' => array(
    'urlDoc' => 'http://js.plus/products/translator/configuration',
    'buttons' => array(array('label' => 'Run translation')),
    'params' => array(
        array(
            'inside' => 'jsplusTranslator',
            'name' => 'provider',
            'default' => 'yandex',
            'type' => 'str',
            'order' => 1000,
            'title' => 'Translation service provider',
            'hint' => '',
            'widget' => 'select',
            'widgetOptions' => array('yandex' => 'Yandex', 'google' => 'Google')
        ),
        array(
            'inside' => 'jsplusTranslator',
            'name' => 'yandexAPIKey',
            'default' => '',
            'type' => 'str',
            'order' => 1010,
            'title' => 'Yandex Translate API key',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'inside' => 'jsplusTranslator',
            'name' => 'googleAPIKey',
            'default' => '',
            'type' => 'str',
            'order' => 1020,
            'title' => 'Google Translate API key',
            'hint' => '',
            'widget' => 'text'
        ),
        array(
            'inside' => 'jsplusTranslator',
            'name' => 'langFrom',
            'default' => 'en',
            'type' => 'str',
            'order' => 1030,
            'title' => '­Language of text for translation',
            'hint' => 'The default language in which the translation is done.',
            'widget' => 'select',
            'widgetOptions' => array('en' => 'English',
                                    'pl' => 'Polish',
                                    'hu' => 'Hungarian',
                                    'ru' => 'Russian',
                                    'uk' => 'Ukraine',
                                    'de' => 'German',
                                    'fr' => 'French',
                                    'be' => 'Belorussian',
                                    'sq' => 'Albanian',
                                    'es' => 'Spanish',
                                    'it' => 'Italian',
                                    'hy' => 'Armenian',
                                    'da' => 'Danish',
                                    'pt' => 'Portuguese',
                                    'sk' => 'Slovak',
                                    'sl' => 'Slovenian',
                                    'nl' => 'Dutch',
                                    'bg' => 'Bulgarian',
                                    'ca' => 'Catalan',
                                    'hr' => 'Croatian',
                                    'cs' => 'Czech',
                                    'el' => 'Greek',
                                    'no' => 'Norwegian',
                                    'mk' => 'Macedonian',
                                    'sv' => 'Swedish',
                                    'fi' => 'Finnish',
                                    'et' => 'Estonian',
                                    'lv' => 'Latvian',
                                    'lt' => 'Lithuanian',
                                    'tr' => 'Turkish',
                                    'ro' => 'Romanian',
                                    'sr' => 'Serbian', 
                                    'az' => 'Azerbaijani')
        ),
        array(
            'inside' => 'jsplusTranslator',
            'name' => 'langTo',
            'default' => 'de',
            'type' => 'str',
            'order' => 1040,
            'title' => '­Language of translation',
            'hint' => '',
            'widget' => 'select',
            'widgetOptions' => array('en' => 'English',
                                    'pl' => 'Polish',
                                    'hu' => 'Hungarian',
                                    'ru' => 'Russian',
                                    'uk' => 'Ukraine',
                                    'de' => 'German',
                                    'fr' => 'French',
                                    'be' => 'Belorussian',
                                    'sq' => 'Albanian',
                                    'es' => 'Spanish',
                                    'it' => 'Italian',
                                    'hy' => 'Armenian',
                                    'da' => 'Danish',
                                    'pt' => 'Portuguese',
                                    'sk' => 'Slovak',
                                    'sl' => 'Slovenian',
                                    'nl' => 'Dutch',
                                    'bg' => 'Bulgarian',
                                    'ca' => 'Catalan',
                                    'hr' => 'Croatian',
                                    'cs' => 'Czech',
                                    'el' => 'Greek',
                                    'no' => 'Norwegian',
                                    'mk' => 'Macedonian',
                                    'sv' => 'Swedish',
                                    'fi' => 'Finnish',
                                    'et' => 'Estonian',
                                    'lv' => 'Latvian',
                                    'lt' => 'Lithuanian',
                                    'tr' => 'Turkish',
                                    'ro' => 'Romanian',
                                    'sr' => 'Serbian', 
                                    'az' => 'Azerbaijani')
        )
    )
),
'jsplusTranslatorReverse' => array(
    'urlDoc' => 'http://js.plus/products/translator/configuration',
    'buttons' => array(array('label' => 'Reverse translation direction'))
),
'jsplusTranslatorConf' => array(
    'urlDoc' => 'http://js.plus/products/translator/configuration',
    'buttons' => array(array('label' => 'Open translation settings dialog'))
)
  );

    /**
     * {@inheritdoc}
     */
    public function getButtons() {
        if (!$this->isInstalled())
            return array();

        $buttons = array();
        foreach ($this->plugins as $pluginName => $pluginDef) {

            if (isset($pluginDef['buttons'])) {
                foreach($pluginDef['buttons'] as $buttonName => $buttonDef) {
                    $image = 'https://cdn.n1ed.com/cdn/buttons/' . $pluginName . '.png';
                    $button = array(
                        'label' => $buttonDef['label'],
                        'image' => $image
                    );
                    $buttons[$pluginName] = $button;
                }
            }

        }
        return $buttons;
    }

  /**
   * {@inheritdoc}
   */
  public function getFile() {
    return '/libraries/jsplusTranslator/plugin.js';
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries(Editor $editor) {
    return array();
  }

  /**
   * {@inheritdoc}
   */
  public function isInternal() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled(Editor $editor) {
    return TRUE;
  }

  function getConfigParam($settings, $param, $default, $type, $inside) {
    $name = $param;
    if (isset($settings[$name]) && strlen($settings[$name]) > 0)
        $value = $settings[$name];
    else
        $value = $default;
    if (isset($type) && $type == 'int') {
        $value = intval($value);
    } else if (isset($type) && $type == 'bool') {
        if ($value == '1')
            $value = true;
        else if ($value == '0')
            $value = false;
    } else if (isset($type) && $type == 'json') {
        $value = json_decode($value);
    }
    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function getConfig(Editor $editor) {
      $settings = array();
      if (isset($editor->getSettings()['plugins']['jsplusTranslator']))
        $settings = $editor->getSettings()['plugins']['jsplusTranslator'];
      $result = array();
      foreach ($this->plugins as $pluginName => $pluginDef) {
          if (isset($pluginDef['params'])) {
              foreach ($pluginDef['params'] as $paramDef) {
                  $value = $this->getConfigParam($settings, $paramDef['name'], $paramDef['default'], $paramDef['type'], isset($paramDef['inside']) ? $paramDef['inside'] : null);
                  if (!array_key_exists('inside', $paramDef))
                      $result[$paramDef['name']] = $value;
                  else {
                      if (!isset($result[$paramDef['inside']]))
                          $result[$paramDef['inside']] = array();
                      $result[$paramDef['inside']][$paramDef['name']] = $value;
                  }
              }
          }
      }
      return $result;
  }

  function addSelectToForm(& $form, $settings, $param, $title, $default, $options, $inside, $urlHelp) {
      $form[$param] = array(
        '#type' => 'select',
        '#title' => $title,
        '#options' => $options,
        '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
        '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
      );
  }

  function addTextboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textfield',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addTextareaToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'textarea',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'str', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  function addCheckboxToForm(& $form, $settings, $param, $title, $default, $desc, $inside, $urlHelp) {
        $form[$param] = array(
          '#type' => 'checkbox',
          '#title' => $title,
          '#default_value' => $this->getConfigParam($settings, $param, $default, 'bool', $inside),
          '#description' => $desc,
          '#attributes' => array('data-url-help' => $urlHelp == '' ? '' : ($urlHelp . '#' . $param), 'data-param-name' => $param)
        );
  }

  public function isInstalled() {
    return file_exists($_SERVER['DOCUMENT_ROOT'] . '/libraries/jsplusTranslator');
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state, Editor $editor) {
    $editor_settings = $editor->getSettings();
    if (isset($editor_settings['plugins']['jsplusTranslator']))
      $settings = $editor_settings['plugins']['jsplusTranslator'];

    $form['#attached']['library'][] = 'ckeditor_translator/ckeditor_translator.admin';

    if (!$this->isInstalled()) {
        $form['warning'] = array(
            '#markup' => 'Looks like CKEditor Translator (Drupal 8 module) is installed but CKEditor add-ons not found. In order to use this module please copy CKEditor add-ons into "libraries" forder in the root of your Drupal 8 installation (create the directory if it does not exist).'
        );
        return $form;
    }

    $params = array();
    foreach ($this->plugins as $pluginName => $pluginDef) {
        if (isset($pluginDef['params']))
            foreach ($pluginDef['params'] as $paramDef) {
                $paramDef['urlDoc'] = $pluginDef['urlDoc'];
                $params[$paramDef['order']] = $paramDef;
            }
    }
    ksort($params); // sort by key (order)

    if (count($params) > 0) {
        foreach ($params as $order => $paramDef) {
            $inside = isset($paramDef['inside']) ? $paramDef['inside'] : null;
            if ($paramDef['type'] == 'json')
                $paramDef['title'] = $paramDef['title'] . "<br/>\nImportant notice: this value needs to be in JSON format. So this is the must to use double quotes (not single ones) for all key names and all string values";
            if ($paramDef['widget'] == 'select') {
                $this->addSelectToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['widgetOptions'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'checkbox') {
                $this->addCheckboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else if ($paramDef['widget'] == 'textarea') {
                $this->addTextareaToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            } else {
                $this->addTextboxToForm($form, $settings, $paramDef['name'], t($paramDef['title']), $paramDef['default'], $paramDef['hint'], $inside, $paramDef['urlDoc']);
            }
        }
    } else {
        //$settings = array();
        $form['warning'] = array(
            '#markup' => 'This add-on does not have parameters.'
        );
    }

    return $form;
  }

}
